# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : task.py
@Date    : 2023/10/31 13:42:42
@Author  : zhchen
@Desc    : 
"""
from urllib.parse import urlencode

from requests import Response

from didabox.utils import MidBox


class TaskBox(MidBox):
    def _task_api(self, data: dict) -> Response:
        response = self.req.post('https://api.dida365.com/api/v2/batch/task', json=data)
        return response

    def add_reminders_task(self, project_id: str, title: str, content: str, trigger_time: str) -> Response:
        """添加任务"""
        trigger_time = self.str2utc(trigger_time)
        json_data = {
            'add': [
                {
                    'items': [],
                    'reminders': [
                        {
                            'id': self.random_int(),
                            'trigger': 'TRIGGER:PT0S',
                        },
                    ],
                    'exDate': [],
                    'dueDate': None,
                    'priority': 0,
                    'isAllDay': False,
                    'repeatFlag': None,
                    'progress': 0,
                    'assignee': None,
                    'sortOrder': -self.random_int(),
                    'startDate': self.shanghai2utc(trigger_time),
                    'isFloating': False,
                    'status': 0,
                    'projectId': project_id,
                    'kind': None,
                    'createdTime': self.shanghai2utc(self.now()),
                    'modifiedTime': self.shanghai2utc(self.now()),
                    'title': title,
                    'tags': [],
                    'timeZone': self.box.req.tz,
                    'content': content,
                    'id': self.random_str(),
                },
            ],
            'update': [],
            'delete': [],
            'addAttachments': [],
            'updateAttachments': [],
            'deleteAttachments': [],
        }
        return self._task_api(json_data)

    def update_task(self, task_info):
        """更新任务"""
        json_data = {
            'add': [],
            'update': [
                task_info,
            ],
            'delete': [],
            'addAttachments': [],
            'updateAttachments': [],
            'deleteAttachments': [],
        }
        return self._task_api(json_data)

    def all_completed(self, from_date='', to_date='', limit=50):
        """
        查询已完成的task
        :param from_date: 格式 2024-03-28 00:57:26
        :param to_date:
        :param limit:
        """
        params = {
            "from": from_date,
            "to": to_date,
            "limit": str(limit)
        }
        # 冒号不能转义 (= =)
        encoded_params = urlencode(params, safe=":")
        response = self.req.get("https://api.dida365.com/api/v2/project/all/completed/?" + encoded_params, params={})
        return response

    def search_all(self, keyword: str):
        """搜索"""
        params = {'keywords': keyword}
        response = self.req.get('https://api.dida365.com/api/v2/search/all', params=params)
        return response



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : utils.py
@Date    : 2023/12/1 9:52:23
@Author  : zhchen
@Desc    : 
"""
import random
import string
from datetime import datetime

import pytz

from didabox.request import Request


class Tools:

    @staticmethod
    def random_str(num=24) -> str:
        # 16进制字符集
        hex_chars = string.hexdigits[:-6]  # 去除字母大小写重复部分
        # 生成随机的16进制字符串
        random_hex_string = ''.join(random.choice(hex_chars) for _ in range(num))
        return random_hex_string

    @staticmethod
    def random_int(num=13) -> int:
        # 生成一个13位随机数字
        random_number = ''.join(str(random.randint(0, 9)) for _ in range(num))
        return int(random_number)

    def shanghai2utc(self, _date: str) -> str:
        """上海时区转成utc时区"""
        dt = datetime.fromisoformat(_date)
        original_timezone = pytz.timezone('Asia/Shanghai')
        dt = original_timezone.localize(dt)
        target_timezone = pytz.timezone('UTC')
        converted_time = dt.astimezone(target_timezone)
        return converted_time.strftime("%Y-%m-%dT%H:%M:%S.%f%z")

    @staticmethod
    def utc2str(_date: str) -> str:
        """[%Y-%m-%dT%H:%M:%S.%f%z]格式转成[%Y-%m-%d %H:%M:%S]格式"""
        dt = datetime.fromisoformat(_date)
        return dt.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def str2utc(_date: str) -> str:
        """[%Y-%m-%d %H:%M:%S]格式转成[%Y-%m-%dT%H:%M:%S.%f%z]格式"""
        dt = datetime.strptime(_date, "%Y-%m-%d %H:%M:%S")
        return dt.strftime("%Y-%m-%dT%H:%M:%S.%f%z")

    @staticmethod
    def now():
        now = datetime.now()
        return now.strftime("%Y-%m-%dT%H:%M:%S.%f%z")


class MidBox(Tools):
    def __init__(self, box):
        from didabox.main import DidaBox
        self.box: DidaBox = box
        self.req: Request = self.box.req



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : login.py
@Date    : 2023/12/1 11:29:05
@Author  : zhchen
@Desc    : 
"""
from didabox.utils import MidBox


class LoginBox(MidBox):

    def sign_on(self, username, password):
        """账号密码登录接口"""
        json_data = {
            "username": username,
            "password": password,
        }
        response = self.req.post("https://api.dida365.com/api/v2/user/signon?wc=true&remember=true", json=json_data)
        self.req.cookies = dict(response.cookies)
        return response




# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : main.py
@Date    : 2023/12/1 10:16:44
@Author  : zhchen
@Desc    : 
"""
import copy

from requests import Response

from didabox.check import CheckBox
from didabox.info import InfoBox
from didabox.pomodoros import PomodorosBox
from didabox.request import Request
from didabox.task import TaskBox
from didabox.login import LoginBox


class DidaBox:
    def __init__(self, cookies: dict, headers=None):
        self.req = Request(cookies=cookies, tz='Asia/Shanghai', headers=headers)
        self.check_box = CheckBox(self)
        self.task_box = TaskBox(self)
        self.login_box = LoginBox(self)
        self.info_box = InfoBox(self)
        self.pomodoros_Box = PomodorosBox(self)

    # == 账号维度 ==
    def check_cookie(self) -> Response:
        """检测cookie是否过期"""
        return self.check_box.check_cookie()

    def sign_on(self, username, password) -> Response:
        """
        账号密码登录, 登录完成后自动更新cookie
        :param username: 账号
        :param password: 密码
        :return:
        """
        return self.login_box.sign_on(username, password)

    # -- 账号维度 --

    # == 清单任务维度 ==
    def base_info(self):
        """清单的基本信息"""
        return self.info_box.base_info()

    def add_simple_task(self, project_id: str, title: str, content: str, trigger_time: str) -> Response:
        """
        添加简单的任务
        :param project_id: 清单id
        :param title: 任务标题
        :param content: 任务内容
        :param trigger_time: 触发时间, 准时触发. 格式: %Y-%m-%d %H:%M:%S
        :return:
        """
        return self.task_box.add_reminders_task(project_id, title, content, trigger_time)

    def get_completed_tasks(self, to_date, from_date: str = '', limit: int = 50) -> Response:
        """
        获取已完成的任务
        :param to_date: 格式 2024-03-28 00:57:26
        :param from_date:
        :param limit:
        """
        return self.task_box.all_completed(from_date=from_date, to_date=to_date, limit=limit)

    def update_content(self, title: str, content: str):
        """
        根据title查询到未完成的任务，将任务内容修改为content
        """
        # 获取所有清单信息
        all_dida_info_res = self.base_info()
        all_dida_info = all_dida_info_res.json()
        result = []
        for row in all_dida_info['syncTaskBean']['update']:
            if row['title'] == title:
                task_info = copy.deepcopy(row)
                task_info['content'] = content
                update_res = self.task_box.update_task(task_info)
                result.append(update_res.json())
        return result

    def search(self, keywords: str) -> Response:
        """根据关键词查询相关任务"""
        return self.task_box.search_all(keywords)
    # -- 清单任务维度 --

    # -- 番茄专注维度 --
    def pomodoros_overview(self) -> Response:
        """获取番茄概览(今天番茄、今天专注时长、总番茄、总专注时长……)"""
        return self.pomodoros_Box.general_for_desktop()

    # -- 番茄专注维度 --



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : request.py
@Date    : 2023/12/1 10:28:52
@Author  : zhchen
@Desc    : 
"""
import requests
from requests import Response


class Request:
    def __init__(self, cookies: dict, tz: str, headers=None):
        """
        简化请求
        :param cookies:
        :param tz: 时区
        :param headers:
        """
        self.cookies = cookies
        self.tz = tz
        self.headers = headers or {
            'authority': 'api.dida365.com',
            'accept': 'application/json, text/plain, */*',
            'accept-language': 'zh-CN,zh;q=0.9',
            'cache-control': 'no-cache',
            'content-type': 'application/json;charset=UTF-8',
            'hl': 'zh_CN',
            'origin': 'https://dida365.com',
            'pragma': 'no-cache',
            'referer': 'https://dida365.com/',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36',
            'x-tz': self.tz,
        }

    def get(self, url, params, **kwargs) -> Response:
        return requests.get(url, params=params, cookies=self.cookies, headers=self.headers, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        return requests.post(url, data, json, cookies=self.cookies, headers=self.headers, **kwargs)



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : pomodoros.py
@Date    : 2025/3/12 16:38:12
@Author  : luke
@Desc    : 
"""
from didabox.utils import MidBox


class PomodorosBox(MidBox):
    def general_for_desktop(self):
        """番茄专注-概览"""
        response = self.req.get(
            'https://api.dida365.com/api/v2/pomodoros/statistics/generalForDesktop',
            params={}
        )
        return response



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : info.py
@Date    : 2024/3/29 13:11:39
@Author  : luke
@Desc    : 
"""

from requests import Response

from didabox.utils import MidBox


class InfoBox(MidBox):

    def base_info(self) -> Response:
        """ 清单的基本信息:
        * 清单: 清单名称、清单id
        * 清单文件夹:
        * 过滤器:
        * 标签:
        * 未完成的任务:
        * ...
        """
        response = self.req.get('https://api.dida365.com/api/v2/batch/check/0', params={})
        return response



# -*- coding: utf-8 -*-

"""
@Project : didabox 
@File    : check.py
@Date    : 2023/10/31 18:03:44
@Author  : zhchen
@Desc    : 
"""
import time

from requests import Response

from didabox.utils import MidBox


class CheckBox(MidBox):

    def check_cookie(self) -> Response:
        """检测cookie是否失效, 失效状态码!=200"""
        # cookie失效例子👇🏻
        # <Response [401]>
        # {"errorId":"w0ht4nm1@ctw6","errorCode":"user_not_sign_on","errorMessage":"user_not_sign_on","data":null}
        params = {'from': str(int(time.time()) * 1000)}
        response = self.req.get('https://api.dida365.com/api/v2/column', params=params)
        return response
